


// Name
//===================================
//
// BlockType
//


// DEPENDENCIES
//===================================

    #include "block.h"
    #include "block_component.h"
    #include "block_condition.h"
    #include "block_type.h"
    #include "collision.h"
    #include <iostream>
    #include "sprite_graphics.h"


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    BlockType::BlockType
    (
        SpriteGraphics* g1,
        SpriteGraphics* g2,
        SpriteGraphics* g3,
        SpriteGraphics* g4,
        std::vector<BlockComponent*> components,
        std::vector<std::vector<BlockCondition*>> conditions
    )
    :
        graphics_ ( { g1, g2, g3, g4 } ),
        components_ ( components ),
        conditions_ ( conditions )
    {};

    BlockType::~BlockType()
    {
        for ( int i = 0; i < NUM_O_MINI_BLOCKS; ++i )
        {
            if ( graphics_[ i ] != nullptr )
            {
                delete graphics_[ i ];
                graphics_[ i ] = nullptr;
            }
        }

        for ( int j = 0; j < components_.size(); ++j )
        {
            if ( components_[ j ] != nullptr )
            {
                delete components_[ j ];
                components_[ j ] = nullptr;
            }
        }

        for ( int k = 0; k < conditions_.size(); ++k )
        {
            for ( int l = 0; l < conditions_[ k ].size(); ++l )
            {
                if ( conditions_[ k ][ l ] != nullptr )
                {
                    delete conditions_[ k ][ l ];
                    conditions_[ k ][ l ] = nullptr;
                }
            }
        }
    };

    void BlockType::interact( Collision& collision, Sprite& sprite, Block& block, Level& level, EventSystem& events, Inventory& inventory, Camera& camera )
    {
        for ( int i = 0; i < components_.size(); ++i )
        {
            bool can_interact = true;

            if ( i < conditions_.size() )
            {
                for ( int j = 0; j < conditions_[ i ].size(); ++j )
                {
                    if ( conditions_[ i ][ j ] != nullptr )
                    {
                        if ( !conditions_[ i ][ j ]->condition( collision, sprite, block ) )
                        {
                            can_interact = false;
                            break;
                        }
                    }
                }
            }

            if ( can_interact )
                if ( components_[ i ] != nullptr )
                    components_[ i ]->interact( collision, sprite, block, level, events, inventory, camera );
        }
    };

    void BlockType::render( Graphics& graphics, Camera& camera, Block& block, bool priority )
    {
        SDL_Rect r = { 0, 0, 8, 8 };

        for ( int i = 0; i < NUM_O_MINI_BLOCKS; ++i )
        {
            int x = ( i % 2 ) * 8;
            int y = floor( i / 2 ) * 8;

            r.x = block.xPixels() + x;
            r.y = block.yPixels() + y;

            if ( graphics_[i] != nullptr )
                graphics_[i]->render( graphics, r, &camera, priority );
        }
    };
