

// Name
//===================================
//
// Game
//


#ifndef GAME_H
#define GAME_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include "game_state.h"
    #include "graphics.h"
    #include "input.h"
    #include <memory>
    #include <vector>


// CLASS
//===================================

    class Game
    {
        public:
            const char* TITLE = "Boskeopolis Land";

            Game();
            ~Game();
            bool running() const;
            void execute();
            void quit();

            void changeState( std::unique_ptr<GameState> state );
            void pushState( std::unique_ptr<GameState> state );
            void popState();

        private:
            Input input_;
            int ticks_ = 0;

            bool running_ = true;
            std::vector< std::unique_ptr<GameState> > states_ = {}; // Polymorphism pointers.
            std::unique_ptr<Graphics> graphics_; // Must be allocated @ runtime, thus must be a pointer.

            int fpsMilliseconds() const;
            bool initSDL();
            void render();
            void firstState();
    };

#endif // GAME_H
