


// Name
//===================================
//
// Inventory
//

#ifndef INVENTORY_H
#define INVENTORY_H


// FORWARD DECLARATIONS
//===================================

    class Graphics;


// DEPENDENCIES
//===================================

    #include "clock.h"
    #include "sprite_graphics.h"
    #include "text.h"
    #include "text_component_marquee.h"
    #include "unit.h"
    #include <vector>


// CLASS
//===================================

    class Inventory
    {
        public:
            Inventory( Clock* clock = new Clock() );
            ~Inventory();

            void update();
            void updateLevelSelect();
            void render( Graphics& graphics );
            void renderLevelSelect( Graphics& graphics );
            void reset();
            void init( Clock* clock = new Clock() );
            void initLevelSelect( bool won = false );

            void addFunds( int n = 100 );
            int funds() const;
            void loseHeart();
            void gainHeart();
            int hearts() const;
            int maxHP() const;
            int clockTime() const;
            Clock* clock();

        private:
            static const int HEARTS_MAX = 3;
            static const int FUNDS_MAX = 99999;
            static const int TOTAL_FUNDS_MAX = 999999999;
            int funds_ = 0;
            int funds_shown_ = 0;
            int total_funds_ = 0;
            int total_funds_shown_ = 0;
            int hearts_ = 3;

            const std::vector<std::string> marquee_messages_ =
            {
                "Kitties Want Answers In Possible Kitty Kibble Shortage",
                "Thank you for reading this message. This message loves you.",
                "Clean up on aisle Delfino.",
                "Experts declare, SimCity 3000 Marquee + platformer = bad game. 9/10.",
                "NEWSFLASH: Crazy woman running round grabbing floating gems everywhere she goes. Details @ 11.",
                "¡AHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH!",
                "WARNING: Playing games for too long can be dangerous. Take 15-minute breaks 'tween each session or we'll come to smash your hand with a hammer-& yes, Jacob, we'll find you @ 2456 Acorn Avenue. We know where you live.",
                "NEWSFLASH: Despite all o' my rage, I am still just a rat in a cage. Details @ 11.",
                "Thou shalt not waste thine time reading these dumb newsflashes 'stead o' playing the game-¡Look out for that Bad Apple! ¡You fool!",
                "If you don't know 'bout the tavern, then it is time."
            };

            static const int VERTICAL_POS = Unit::WINDOW_HEIGHT_PIXELS - 24;

            SDL_Rect bg_pos_ = { 0, Unit::WINDOW_HEIGHT_PIXELS - 32, Unit::WINDOW_WIDTH_PIXELS, 32 };

            static const int FUNDS_X = 16;
            SpriteGraphics gem_icon_gfx_ = SpriteGraphics( Graphics::SpriteSheet::CHARSET, 32, 16 );
            SDL_Rect gem_icon_pos_ = { 8, VERTICAL_POS, 8, 8 };

            SpriteGraphics heart_icon_gfx_ = SpriteGraphics( Graphics::SpriteSheet::LVTILESET_UNIVERSAL, 32, 8 );
            SDL_Rect heart_icon_pos_ = { 80, VERTICAL_POS, 8, 8 };

            //Text funds_string_ = Text( "DEMO", 16, VERTICAL_POS );

            Text marquee_ = Text( "Kitties Want Answers In Possible Kitty Kibble Shortage", Unit::WINDOW_WIDTH_PIXELS, Unit::WINDOW_HEIGHT_PIXELS - 8, Text::FontShade::BLACK, Text::FontAlign::FA_LEFT, false, new TextComponentMarquee( Direction::Horizontal::LEFT, 1 ) );

            static const int CLOCK_X = 120;
            SpriteGraphics clock_gfx_ = SpriteGraphics( Graphics::SpriteSheet::LVTILESET_UNIVERSAL, 40, 0 );
            SDL_Rect clock_gfx_pos_ = { 112, VERTICAL_POS, 8, 8 };
            //Text clock_text_ = Text( "DEMO", 120, VERTICAL_POS );
            Clock* clock_ = nullptr;

            static const int VERTICAL_POS_LEVEL_SELECT = Unit::WINDOW_HEIGHT_PIXELS - 16;
            SDL_Rect lv_select_gem_icon_pos_ = { 8, VERTICAL_POS_LEVEL_SELECT, 8, 8 };

            std::string randomMarqueeMessage() const;
    };

#endif // INVENTORY_H

