

// Name
//===================================
//
// Map
//

#ifndef MAP_H
#define MAP_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include "background.h"
    #include "palette.h"
    #include <vector>
    #include "warp.h"


// CLASS
//===================================

    class Map
    {
        public:
            Map
            (
                std::vector<int> blocks,
                std::vector<int> sprites,
                int width,
                int height,
                Palette::PaletteSet palette = { Palette::PaletteType::CLASSIC_GREEN, 2 },
                std::vector<Background> backgrounds = { Background() },
                std::vector<Warp> warps = {}
            );
            ~Map();

            int widthBlocks() const;
            int heightBlocks() const;
            int widthPixels() const;
            int heightPixels() const;
            int blocksSize() const;
            int spritesSize() const;
            int block( int n ) const;
            int sprite( int n ) const;
            int mapX( int n ) const;
            int mapY( int n ) const;
            const bool changed() const;
            void changeBlock( int where, int value );
            void deleteBlock( int where );
            void update();
            Palette::PaletteSet palette() const;
            void renderBG( Graphics& graphics, Camera& camera );
            Warp* getWarp( int x_sub_pixels, int y_sub_pixels );

        private:
            std::vector<int> blocks_;
            std::vector<int> sprites_;
            std::vector<Warp> warps_;
            Palette::PaletteSet palette_;
            std::vector<Background> backgrounds_;
            int width_;
            int height_;
            bool changed_ = true;

            bool inBounds( int n ) const;
    };

#endif // MAP_H
