
// Name
//===================================
//
// SpriteGraphics
//

#ifndef SPRITE_GRAPHICS_H
#define SPRITE_GRAPHICS_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class Sprite;


// DEPENDENCIES
//===================================

    #include "graphics.h"


// CLASS
//===================================

    class SpriteGraphics
    {
        public:
            enum SpriteRotation
            {
                SPROT_NONE = 0,
                SPROT_CLOCKWISE = 90,
                SPROT_180 = 180,
                SPROT_COUNTERCLOCKWISE = 270
            };

            SpriteGraphics
            (
                Graphics::SpriteSheet texture = Graphics::SpriteSheet::LIMIT,
                int current_frame_x = 0,
                int current_frame_y = 0,
                bool flip_x = false,
                bool flip_y = false,
                double rotation = SpriteRotation::SPROT_NONE,
                bool priority = false
            );

            bool flip_x_ = false;
            bool flip_y_ = false;
            double rotation_ = 0;
            bool priority_ = false;
            int current_frame_x_ = 0;
            int current_frame_y_ = 0;

            virtual ~SpriteGraphics();
            virtual void update( Sprite& sprite );
            void render( Graphics& graphics, SDL_Rect bound_box, Camera* camera, bool priority = false );
            void adjustBoundBox( SDL_Rect new_box );
            bool visible_ = true;

        protected:
            Graphics::SpriteSheet texture_ = Graphics::SpriteSheet::LIMIT;
            int prev_frame_x_ = 0;
            int prev_frame_y_ = 0;

            int cutoffLeft  ( int object_x, int camera_x ) const;
            int cutoffTop   ( int object_y, int camera_y ) const;
            int cutoffRight ( int object_x, int camera_x, int object_w, int camera_w ) const;
            int cutoffBottom( int object_y, int camera_y, int object_h, int camera_h ) const;
            int cutoffLeft  ( SDL_Rect& obj_r, Camera& c ) const;
            int cutoffTop   ( SDL_Rect& obj_r, Camera& c ) const;
            int cutoffRight ( SDL_Rect& obj_r, Camera& c ) const;
            int cutoffBottom( SDL_Rect& obj_r, Camera& c ) const;
    };


#endif // SPRITE_GRAPHICS_H

