


// Name
//===================================
//
// SpriteSystem
//


// DEPENDENCIES
//===================================

    #include "block_system.h"
    #include "camera.h"
    #include "hydrant_graphics.h"
    #include "hydrant_movement.h"
    #include "map.h"
    #include "player_graphics.h"
    #include "player_movement.h"
    #include "sprite_system.h"


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    SpriteSystem::SpriteSystem( int entrance_x, int entrance_y )
    :
        hero_ ( Sprite( new PlayerGraphics(), new PlayerMovement(), entrance_x, entrance_y, 16, 24, Sprite::SpriteType::HERO ) )
    {};

    SpriteSystem::~SpriteSystem() {};

    Sprite* SpriteSystem::spriteType( int type, int x, int y )
    {
        switch ( type )
        {
            default:
                return new Sprite( new HydrantGraphics(), new HydrantMovement(), Unit::SubPixelsToPixels( x ), Unit::SubPixelsToPixels( y ), 16, 16, Sprite::SpriteType::ENEMY, 100, 1500, 3000 );
        }
    };

    void SpriteSystem::spritesFromMap( Map& lvmap )
    {
        int x = 0;
        int y = 0;

        for ( int i = 0; i < lvmap.spritesSize(); ++i )
        {
            x = Unit::BlocksToSubPixels( lvmap.mapX( i ) );
            y = Unit::BlocksToSubPixels( lvmap.mapY( i ) );

            const int type = lvmap.sprite( i ) - 1;

            if ( type > -1 )
            {
                sprites_.push_back( spriteType( type, x, y ) );
            }
        }
    };

    void SpriteSystem::interact( BlockSystem& blocks, Level& level, EventSystem& events, Inventory& inventory, Camera& camera )
    {
        for ( int i = 0; i < sprites_.size(); ++i )
        {
            if ( sprites_[ i ] != nullptr )
                if ( sprites_[ i ]->interact_ )
                {
                    blocks.interact( *sprites_[ i ], level, events, inventory, camera );
                }
        }
        blocks.interact( hero_, level, events, inventory, camera );
    };

    void SpriteSystem::reset( Map& lvmap )
    {
        clearSprites();
        spritesFromMap( lvmap );
    };

    void SpriteSystem::clearSprites()
    {
        for ( int i = 0; i < sprites_.size(); ++i )
        {
            if ( sprites_[ i ] != nullptr )
            {
                delete sprites_[ i ];
                sprites_[ i ] = nullptr;
            }
        }
        sprites_.clear();
    };

    void SpriteSystem::update( Input& input, Camera& camera, Map& lvmap, Game& game, EventSystem& events )
    {
        for ( int i = 0; i < sprites_.size(); ++i )
        {
            if ( sprites_[ i ] != nullptr )
            {
                if ( camera.onscreen( sprites_[ i ]->boundBox() ) )
                    sprites_[ i ]->update( input, camera, lvmap, game, events );
            }
        }
        hero_.update( input, camera, lvmap, game, events );

        spriteInteraction();
    };

    void SpriteSystem::spriteInteraction()
    {
        for ( int i = 0; i < sprites_.size(); ++i )
        {
            if ( sprites_[ i ] != nullptr )
            {
                sprites_[ i ]->interact( hero_ );
                hero_.interact( *sprites_[ i ] );

                for ( int j = 0; j < sprites_.size(); ++j )
                {
                    if ( sprites_[ j ] != nullptr )
                        if ( i != j )
                            sprites_[ i ]->interact( *sprites_[ j ] );
                }
            }
        }
    };

    void SpriteSystem::render( Graphics& graphics, Camera& camera )
    {
        for ( int i = 0; i < sprites_.size(); ++i )
        {
            if ( sprites_[ i ] != nullptr )
            {
                sprites_[ i ]->render( graphics, camera );
            }
        }
        hero_.render( graphics, camera );
    };

    Sprite& SpriteSystem::hero()
    {
        return hero_;
    };
